@@ -13,6 +13,7 @@
 import weakref
 
 from requests.adapters import BaseAdapter
+from requests.utils import requote_uri
 import six
 from six.moves.urllib import parse as urlparse
 
@@ -62,6 +63,9 @@ class _RequestHistoryTracker(object):
     def call_count(self):
         return len(self.request_history)
 
+    def reset(self):
+        self.request_history = []
+
 
 class _RunRealHTTP(Exception):
     """A fake exception to jump out of mocking and allow a real request.
@@ -99,7 +103,7 @@ class _Matcher(_RequestHistoryTracker):
             url_parts = urlparse.urlparse(url)
             self._scheme = url_parts.scheme.lower()
             self._netloc = url_parts.netloc.lower()
-            self._path = url_parts.path or '/'
+            self._path = requote_uri(url_parts.path or '/')
             self._query = url_parts.query
 
             if not case_sensitive:
@@ -249,7 +253,9 @@ class Adapter(BaseAdapter, _RequestHistoryTracker):
             if resp is not None:
                 request._matcher = weakref.ref(matcher)
                 resp.connection = self
-                logger.debug('{} {} {}'.format(request._request.method,request._request.url, resp.status_code))
+                logger.debug('{} {} {}'.format(request._request.method,
+                                               request._request.url,
+                                               resp.status_code))
                 return resp
 
         raise exceptions.NoMockAddress(request)
@@ -305,5 +311,10 @@ class Adapter(BaseAdapter, _RequestHistoryTracker):
         """
         self._matchers.append(matcher)
 
+    def reset(self):
+        super(Adapter, self).reset()
+        for matcher in self._matchers:
+            matcher.reset()
+
 
 __all__ = ['Adapter']
