@@ -49,6 +49,69 @@ class MockerTests(base.TestCase):
         self.assertMockStopped()
         mocker.stop()
 
+    def test_with_session(self):
+        url = 'http://test.url/path'
+        url_inner = 'http://test.url/inner'
+        url_outer = 'http://test.url/outer'
+        with requests_mock.Mocker() as global_mock:
+            global_mock.get(url_outer, text='global')
+
+            session_a = requests.Session()
+            session_b = requests.Session()
+
+            session_a_original_send = session_a.send
+            session_b_original_send = session_b.send
+            self.assertNotEqual(session_a_original_send,
+                                session_b_original_send)
+
+            mocker_a = requests_mock.Mocker(session=session_a)
+            mocker_b = requests_mock.Mocker(session=session_b)
+
+            mocker_a.start()
+            mocker_b.start()
+
+            mocker_a.register_uri('GET', url, text='resp_a')
+            mocker_a.register_uri('GET', url_outer, real_http=True)
+            mocker_b.register_uri('GET', url, text='resp_b')
+
+            with requests_mock.Mocker(session=session_b) as mocker_b_inner:
+                mocker_b_inner.register_uri('GET', url, real_http=True)
+                mocker_b_inner.register_uri('GET',
+                                            url_inner,
+                                            text='resp_b_inner')
+
+                self.assertEqual('resp_a', session_a.get(url).text)
+                self.assertEqual('resp_b', session_b.get(url).text)
+                self.assertRaises(exceptions.NoMockAddress,
+                                  session_a.get,
+                                  url_inner)
+                self.assertEqual('resp_b_inner', session_b.get(url_inner).text)
+
+            self.assertEqual('resp_a', session_a.get(url).text)
+            self.assertEqual('resp_b', session_b.get(url).text)
+            self.assertRaises(exceptions.NoMockAddress,
+                              session_a.get,
+                              url_inner)
+            self.assertRaises(exceptions.NoMockAddress,
+                              session_b.get,
+                              url_inner)
+            self.assertEqual('global', session_a.get(url_outer).text)
+            self.assertRaises(exceptions.NoMockAddress,
+                              session_b.get,
+                              url_outer)
+
+            self.assertNotEqual(session_a.send, session_a_original_send)
+            self.assertNotEqual(session_b.send, session_b_original_send)
+            self.assertNotEqual(session_a.send, session_b.send)
+
+            mocker_a.stop()
+            mocker_b.stop()
+
+            self.assertEqual(session_a.send, session_a_original_send)
+            self.assertEqual(session_b.send, session_b_original_send)
+
+        self.assertEqual(requests.Session.send, original_send)
+
     def test_with_context_manager(self):
         self.assertMockStopped()
         with requests_mock.Mocker() as m:
@@ -76,6 +139,58 @@ class MockerTests(base.TestCase):
         self.assertEqual(test_text, resp.text)
         self.assertEqual(test_bytes, resp.content)
 
+    @mock.patch('requests.adapters.HTTPAdapter.send')
+    def test_real_http_changes(self, real_send):
+        url = 'http://www.google.com/'
+        test_text = 'real http data'
+        test_bytes = test_text.encode('utf-8')
+
+        req = requests.Request(method='GET', url=url)
+        real_send.return_value = response.create_response(req.prepare(),
+                                                          status_code=200,
+                                                          content=test_bytes)
+
+        with requests_mock.Mocker() as m:
+            # real_http defaults to false so should raise NoMockAddress
+
+            self.assertRaises(exceptions.NoMockAddress,
+                              requests.get,
+                              url)
+
+            self.assertEqual(1, m.call_count)
+            self.assertEqual(0, real_send.call_count)
+
+            # change the value of real_http mid test
+            m.real_http = True
+
+            # fetch the url again and it should go through to the real url that
+            # we've mocked out at a lower level.
+            resp = requests.get(url)
+
+        self.assertEqual(1, real_send.call_count)
+        self.assertEqual(url, real_send.call_args[0][0].url)
+
+        self.assertEqual(test_text, resp.text)
+        self.assertEqual(test_bytes, resp.content)
+
+    @mock.patch('requests.adapters.HTTPAdapter.send')
+    def test_real_http_and_session(self, real_send):
+        url = 'http://www.google.com/'
+        test_text = 'real http data'
+        test_bytes = test_text.encode('utf-8')
+
+        req = requests.Request(method='GET', url=url)
+        real_send.return_value = response.create_response(req.prepare(),
+                                                          status_code=200,
+                                                          content=test_bytes)
+
+        session = requests.Session()
+        with requests_mock.Mocker(session=session, real_http=True):
+            resp = session.get(url)
+
+        self.assertEqual(test_text, resp.text)
+        self.assertEqual(test_bytes, resp.content)
+
     @requests_mock.mock()
     def test_with_test_decorator(self, m):
         self._do_test(m)
@@ -207,7 +322,19 @@ class MockerTests(base.TestCase):
         copy_of_mocker = mocker.copy()
         self.assertIsNot(copy_of_mocker, mocker)
         self.assertEqual(copy_of_mocker._kw, mocker._kw)
-        self.assertEqual(copy_of_mocker._real_http, mocker._real_http)
+        self.assertEqual(copy_of_mocker.real_http, mocker.real_http)
+
+    @requests_mock.mock()
+    def test_reset_mock_reverts_call_count(self, request_mock):
+        url = 'http://test.url/path'
+        request_mock.get(url, text='resp')
+        requests.get(url)
+
+        self.assertEqual(request_mock.call_count, 1)
+
+        # reset count and verify it is 0
+        request_mock.reset_mock()
+        self.assertEqual(request_mock.call_count, 0)
 
 
 class MockerHttpMethodsTests(base.TestCase):
@@ -460,3 +587,16 @@ class MockerHttpMethodsTests(base.TestCase):
         self.assertEqual(text, new_resp.text)
         self.assertIsInstance(orig_resp.request.matcher, adapter._Matcher)
         self.assertIsNone(new_resp.request.matcher)
+
+    @requests_mock.mock()
+    def test_stream_zero_bytes(self, m):
+        content = b'blah'
+
+        m.get("http://test", content=content)
+        res = requests.get("http://test", stream=True)
+        zero_val = res.raw.read(0)
+        self.assertEqual(b'', zero_val)
+        self.assertFalse(res.raw.closed)
+
+        full_val = res.raw.read()
+        self.assertEqual(content, full_val)
